@@ -67,17 +67,32 @@
 extern char NoName[];
 extern int  resizeWhenAdd;
 
+extern int GetGravityOffsets ();
 extern TwmWindow *AddWindow();
 extern int MappedNotOverride();
+extern AddDefaultBindings ();
 extern void GrabButtons();
 extern void GrabKeys();
 extern void UngrabButtons();
 extern void UngrabKeys();
 extern void GetWindowSizeHints();
+extern int AnimateButton ();
+extern int AnimateHighlight ();
+extern void CreateWindowRegions ();
+extern Bool PlaceWindowInRegion ();
+extern void RemoveWindowFromRegion ();
+extern name_list **AddWindowRegion ();
 extern int AddingX;	
 extern int AddingY;
 extern int AddingW;
 extern int AddingH;
 
+#ifndef NO_LOCALE
+extern char *GetWMPropertyString();
+#endif /* NO_LOCALE */
+
+extern SetHighlightPixmap ();
+extern FetchWmColormapWindows ();
+extern FetchWmProtocols ();
 #endif /* _ADD_WINDOW_ */
 
