@@ -130,7 +130,7 @@ static FILE *start_m4();
 static char *m4_defs();
 #endif
 
-extern int yylineno;
+extern int twmrc_lineno;
 extern int mods;
 
 int ConstrainedMoveTime = 400;		/* milliseconds, event times */
@@ -172,7 +172,7 @@ static int doparse (ifunc, srctypename, srcname)
     mods = 0;
     ptr = 0;
     len = 0;
-    yylineno = 1;
+    twmrc_lineno = 0;
     ParseError = FALSE;
     twmInputFunc = ifunc;
     overflowlen = 0;
@@ -410,7 +410,7 @@ static int twmFileInput()
             if (fgets(buff, BUF_LEN, rc_includes[include_file].fp) == NULL) {
                 free(rc_includes[include_file].name);
                 fclose(rc_includes[include_file].fp);
-                yylineno = rc_includes[include_file--].lineno;
+                twmrc_lineno = rc_includes[include_file--].lineno;
             } else
                 break;
         }
@@ -418,7 +418,7 @@ static int twmFileInput()
         if (!include_file)
 	if (fgets(buff, BUF_LEN, twmrc) == NULL)
 	    return 0;
-	yylineno++;
+	twmrc_lineno++;
 
         if (strncmp(buff, "include", 7) == 0) {
              /* Whoops, an include file! */
@@ -441,8 +441,8 @@ static int twmFileInput()
                   continue;
              }
              rc_includes[include_file].fp = fp;
-             rc_includes[include_file].lineno = yylineno;
-             yylineno = 0;
+             rc_includes[include_file].lineno = twmrc_lineno;
+             twmrc_lineno = 0;
              rc_includes[include_file].name = malloc(strlen(p)+1);
              strcpy(rc_includes[include_file].name, p);
              continue;
@@ -458,16 +458,39 @@ static int twmFileInput()
 
 static int m4twmFileInput()
 {
-    int pppp;
+    int line;
+    extern char *keepM4_filename;
+    static FILE *cp = NULL;
+
+    if ( cp == NULL && keepM4_filename ) {
+      cp = fopen (keepM4_filename,"w"); 
+      if ( cp == NULL ) {
+	fprintf (stderr,
+		 "%s:  unable to create m4 output %s, ignoring\n",
+		 ProgramName, keepM4_filename);
+	keepM4_filename = NULL;
+      }
+    }
+
     if (overflowlen){
 	return((int) overflowbuff[--overflowlen]);
     }
 
     while (ptr == len) {
-	if (fgets(buff, BUF_LEN, twmrc) == NULL) return(0);
-	
-	yylineno++;
-	
+	nextline:
+	if (fgets(buff, BUF_LEN, twmrc) == NULL) {
+	  if ( cp ) fclose (cp);
+	  return(0);
+	}
+	if ( cp ) fputs (buff, cp);
+
+	if (sscanf(buff, "#line %d", &line)) {
+	    twmrc_lineno = line - 1;
+	    goto nextline;
+	} else {
+	    twmrc_lineno++;
+	}
+
 	ptr = 0;
 	len = strlen(buff);
     }
@@ -606,6 +629,8 @@ typedef struct _TwmKeyword {
 #define kw0_IgnoreLockModifier		59
 #define kw0_AutoFocusToTransients       60 /* kai */
 #define kw0_PackNewWindows		61
+#define kw0_IgnoreCaseInMenuSelection	62
+#define kw0_SloppyFocus                 63
 
 #define kws_UsePPosition		1
 #define kws_IconFont			2
@@ -661,6 +686,11 @@ typedef struct _TwmKeyword {
 #define kwn_OpenWindowTimeout		31
 #define kwn_RaiseOnClickButton		32
 
+#define kwn_BorderTop			33
+#define kwn_BorderBottom		34
+#define kwn_BorderLeft			35
+#define kwn_BorderRight			36
+
 #define kwcl_BorderColor		1
 #define kwcl_IconManagerHighlight	2
 #define kwcl_BorderTileForeground	3
@@ -703,11 +733,15 @@ static TwmKeyword keytable[] = {
     { "autorelativeresize",	KEYWORD, kw0_AutoRelativeResize },
     { "autosqueeze",		AUTOSQUEEZE, 0 },
     { "benicetocolormap",	KEYWORD, kw0_BeNiceToColormap },
+    { "borderbottom",		NKEYWORD, kwn_BorderBottom },
     { "bordercolor",		CLKEYWORD, kwcl_BorderColor },
+    { "borderleft",		NKEYWORD, kwn_BorderLeft },
     { "borderresizecursors",	KEYWORD, kw0_BorderResizeCursors },
+    { "borderright",		NKEYWORD, kwn_BorderRight },
     { "bordershadowdepth",	NKEYWORD, kwn_BorderShadowDepth },
     { "bordertilebackground",	CLKEYWORD, kwcl_BorderTileBackground },
     { "bordertileforeground",	CLKEYWORD, kwcl_BorderTileForeground },
+    { "bordertop",		NKEYWORD, kwn_BorderTop },
     { "borderwidth",		NKEYWORD, kwn_BorderWidth },
     { "button",			BUTTON, 0 },
     { "buttonindent",		NKEYWORD, kwn_ButtonIndent },
@@ -732,6 +766,7 @@ static TwmKeyword keytable[] = {
     { "donticonifybyunmapping",	DONT_ICONIFY_BY_UNMAPPING, 0 },
     { "dontmoveoff",		KEYWORD, kw0_DontMoveOff },
     { "dontpaintrootwindow",	KEYWORD, kw0_DontPaintRootWindow },
+    { "dontsave",		DONT_SAVE, 0 },
     { "dontsetinactive",	DONTSETINACTIVE, 0 },
     { "dontsqueezetitle",	DONT_SQUEEZE_TITLE, 0 },
     { "dontwarpcursorinwmap",	KEYWORD, kw0_DontWarpCursorInWMap },
@@ -762,6 +797,7 @@ static TwmKeyword keytable[] = {
     { "f.exec",			FSKEYWORD, F_EXEC },
     { "f.file",			FSKEYWORD, F_FILE },
     { "f.fill",			FSKEYWORD, F_FILL },
+    { "f.fittocontent",		FKEYWORD, F_FITTOCONTENT },
     { "f.focus",		FKEYWORD, F_FOCUS },
     { "f.forcemove",		FKEYWORD, F_FORCEMOVE },
     { "f.forwiconmgr",		FKEYWORD, F_FORWICONMGR },
@@ -880,12 +916,15 @@ static TwmKeyword keytable[] = {
     { "iconmanagers",		ICONMGRS, 0 },
     { "iconmanagershadowdepth",	NKEYWORD, kwn_IconManagerShadowDepth },
     { "iconmanagershow",	ICONMGR_SHOW, 0 },
+    { "iconmenudontshow",	ICONMENU_DONTSHOW, 0 },
     { "iconmgr",		ICONMGR, 0 },
     { "iconregion",		ICON_REGION, 0 },
     { "iconregionalignement",	SKEYWORD, kws_IconRegionAlignement },
     { "iconregionjustification",SKEYWORD, kws_IconRegionJustification },
     { "icons",			ICONS, 0 },
+    { "ignorecaseinmenuselection",	KEYWORD, kw0_IgnoreCaseInMenuSelection },
     { "ignorelockmodifier",	KEYWORD, kw0_IgnoreLockModifier },
+    { "ignoremodifier",		IGNOREMODIFIER, 0 },
     { "interpolatemenucolors",	KEYWORD, kw0_InterpolateMenuColors },
     { "l",			LOCK, 0 },
     { "left",			JKEYWORD, J_LEFT },
@@ -970,6 +1009,7 @@ static TwmKeyword keytable[] = {
     { "shortallwindowsmenus",	KEYWORD, kw0_ShortAllWindowsMenus },
     { "showiconmanager",	KEYWORD, kw0_ShowIconManager },
     { "showworkspacemanager",	KEYWORD, kw0_ShowWorkspaceManager },
+    { "sloppyfocus",            KEYWORD, kw0_SloppyFocus },
     { "smarticonify",		KEYWORD, kw0_SmartIconify },
     { "sorticonmanager",	KEYWORD, kw0_SortIconManager },
 #ifdef SOUNDS
@@ -1014,7 +1054,9 @@ static TwmKeyword keytable[] = {
     { "warpunmapped",		KEYWORD, kw0_WarpUnmapped },
     { "west",			DKEYWORD, D_WEST },
     { "window",			WINDOW, 0 },
+    { "windowbox",		WINDOW_BOX, 0 },
     { "windowfunction",		WINDOW_FUNCTION, 0 },
+    { "windowgeometries",	WINDOW_GEOMETRIES, 0 },
     { "windowregion",		WINDOW_REGION, 0 },
     { "windowring",		WINDOW_RING, 0 },
     { "windowringexclude",      WINDOW_RING_EXCLUDE, 0},
@@ -1302,6 +1344,14 @@ int do_single_keyword (keyword)
 	Scr->PackNewWindows = TRUE;
 	return 1;
 
+      case kw0_IgnoreCaseInMenuSelection:
+	Scr->IgnoreCaseInMenuSelection = TRUE;
+	return 1;
+
+      case kw0_SloppyFocus:
+	Scr->SloppyFocus = TRUE;
+	return 1;
+
     }
     return 0;
 }
@@ -1618,6 +1668,26 @@ int do_number_keyword (keyword, num)
 	if (Scr->BorderShadowDepth < 0) Scr->BorderShadowDepth = 2;
 	return 1;
 
+      case kwn_BorderLeft:
+	if (Scr->FirstTime) Scr->BorderLeft = num;
+	if (Scr->BorderLeft < 0) Scr->BorderLeft = 0;
+	return 1;
+
+      case kwn_BorderRight:
+	if (Scr->FirstTime) Scr->BorderRight = num;
+	if (Scr->BorderRight < 0) Scr->BorderRight = 0;
+	return 1;
+
+      case kwn_BorderTop:
+	if (Scr->FirstTime) Scr->BorderTop = num;
+	if (Scr->BorderTop < 0) Scr->BorderTop = 0;
+	return 1;
+
+      case kwn_BorderBottom:
+	if (Scr->FirstTime) Scr->BorderBottom = num;
+	if (Scr->BorderBottom < 0) Scr->BorderBottom = 0;
+	return 1;
+
       case kwn_TitleButtonShadowDepth:
 	if (Scr->FirstTime) Scr->TitleButtonShadowDepth = num;
 	if (Scr->TitleButtonShadowDepth < 0) Scr->TitleButtonShadowDepth = 2;
@@ -2027,7 +2097,7 @@ FILE *fraw;
                 dup2(fids[1], 1);       /* stdout = pipe to parent */
                 /* get_defs("m4", dpy, display_name) */
                 tmp_file = m4_defs(dpy, display_name);
-                execlp("m4", "m4", tmp_file, "-", NULL);
+                execlp("m4", "m4", "-s", tmp_file, "-", NULL);
                 /* If we get here we are screwed... */
                 perror("Can't execlp() m4");
                 exit(124);
